@@ -558,7 +558,8 @@ def use_reduce(depstr, uselist=[], masklist=[], matchall=False, excludeall=[], i
 						stack[level].extend(l)
 					continue
 
-				if stack[level]:
+				if stack[level] and isinstance(stack[level][-1],
+					basestring):
 					if stack[level][-1] == "||" and not l:
 						#Optimize: || ( ) -> .
 						stack[level].pop()
@@ -583,7 +584,8 @@ def use_reduce(depstr, uselist=[], masklist=[], matchall=False, excludeall=[], i
 					#ends in a non-operator. This is almost equivalent to stack[level][-1]=="||",
 					#expect that it skips empty levels.
 					while k>=0:
-						if stack[k]:
+						if stack[k] and isinstance(stack[k][-1],
+							basestring):
 							if stack[k][-1] == "||":
 								return k
 							elif stack[k][-1][-1] != "?":
@@ -1172,6 +1174,12 @@ class Atom(_unicode):
 	class emulates most of the str methods that are useful with atoms.
 	"""
 
+	# Distiguishes package atoms from other atom types
+	package = True
+
+	# Distiguishes soname atoms from other atom types
+	soname = False
+
 	class _blocker(object):
 		__slots__ = ("overlap",)
 
@@ -1561,6 +1569,25 @@ class Atom(_unicode):
 		memo[id(self)] = self
 		return self
 
+	def match(self, pkg):
+		"""
+		Check if the given package instance matches this atom. This
+		includes support for virtual matches via PROVIDE metadata.
+
+		@param pkg: a Package instance
+		@type pkg: Package
+		@return: True if this atom matches pkg, otherwise False
+		@rtype: bool
+		"""
+		if pkg.cp == self.cp:
+			return bool(match_from_list(self, [pkg]))
+		else:
+			for provided_cp in pkg.provided_cps:
+				if provided_cp == self.cp:
+					return bool(match_from_list(
+						self.replace(self.cp, provided_cp, 1), [pkg]))
+		return False
+
 _extended_cp_re_cache = {}
 
 def extended_cp_match(extended_cp, other_cp):
