@@ -11,12 +11,37 @@ from portage import _os_merge
 from portage import _unicode_decode
 from portage import _unicode_encode
 from portage.cache.mappings import slot_dict_class
-from portage.exception import CommandNotFound
+from portage.exception import CommandNotFound, InvalidData
 from portage.localization import _
 from portage.util import getlibpaths
 from portage.util import grabfile
 from portage.util import normalize_path
+from portage.util import varexpand
 from portage.util import writemsg_level
+from portage.util._dyn_libs.NeededEntry import NeededEntry
+
+# Map ELF e_machine values from NEEDED.ELF.2 to approximate multilib
+# categories. This approximation will produce incorrect results on x32
+# and mips systems, but the result is not worse than using the raw
+# e_machine value which was used by earlier versions of portage.
+_approx_multilib_categories = {
+	"386":           "x86_32",
+	"68K":           "m68k_32",
+	"AARCH64":       "arm_64",
+	"ALPHA":         "alpha_64",
+	"ARM":           "arm_32",
+	"IA_64":         "ia_64",
+	"MIPS":          "mips_o32",
+	"PARISC":        "hppa_64",
+	"PPC":           "ppc_32",
+	"PPC64":         "ppc_64",
+	"S390":          "s390_64",
+	"SH":            "sh_32",
+	"SPARC":         "sparc_32",
+	"SPARC32PLUS":   "sparc_32",
+	"SPARCV9":       "sparc_64",
+	"X86_64":        "x86_64",
+}
 
 class LinkageMapELF(object):
 
@@ -294,21 +319,31 @@ class LinkageMapELF(object):
 					"in %s: %s\n\n") % (location, l),
 					level=logging.ERROR, noiselevel=-1)
 				continue
-			fields = l.split(";")
-			if len(fields) < 5:
-				writemsg_level(_("\nWrong number of fields " \
-					"in %s: %s\n\n") % (location, l),
+			try:
+				entry = NeededEntry.parse(location, l)
+			except InvalidData as e:
+				writemsg_level("\n%s\n\n" % (e,),
 					level=logging.ERROR, noiselevel=-1)
 				continue
-			arch = fields[0]
-			obj = fields[1]
-			soname = fields[2]
-			path = frozenset(normalize_path(x) \
-				for x in filter(None, fields[3].replace(
-				"${ORIGIN}", os.path.dirname(obj)).replace(
-				"$ORIGIN", os.path.dirname(obj)).split(":")))
+
+			# If NEEDED.ELF.2 contains the new multilib category field,
+			# then use that for categorization. Otherwise, if a mapping
+			# exists, map e_machine (entry.arch) to an approximate
+			# multilib category. If all else fails, use e_machine, just
+			# as older versions of portage did.
+			arch = entry.multilib_category
+			if arch is None:
+				arch = _approx_multilib_categories.get(
+					entry.arch, entry.arch)
+
+			obj = entry.filename
+			soname = entry.soname
+			expand = {"ORIGIN": os.path.dirname(entry.filename)}
+			path = frozenset(normalize_path(varexpand(x, expand))
+				for x in entry.runpaths)
 			path = frozensets.setdefault(path, path)
-			needed = frozenset(x for x in fields[4].split(",") if x)
+			needed = frozenset(entry.needed)
+
 			needed = frozensets.setdefault(needed, needed)
 
 			obj_key = self._obj_key(obj)
@@ -678,7 +713,7 @@ class LinkageMapELF(object):
 						rValue[soname].add(provider)
 		return rValue
 
-	def findConsumers(self, obj, exclude_providers=None):
+	def findConsumers(self, obj, exclude_providers=None, greedy=True):
 		"""
 		Find consumers of an object or object key.
 
@@ -715,6 +750,9 @@ class LinkageMapELF(object):
 			'/usr/lib/libssl.so.0.9.8'), and return True if the library is
 			owned by a provider which is planned for removal.
 		@type exclude_providers: collection
+		@param greedy: If True, then include consumers that are satisfied
+		by alternative providers, otherwise omit them. Default is True.
+		@type greedy: Boolean
 		@rtype: set of strings (example: set(['/bin/foo', '/usr/bin/bar']))
 		@return: The return value is a soname -> set-of-library-paths, where
 		set-of-library-paths satisfy soname.
@@ -769,16 +807,19 @@ class LinkageMapELF(object):
 		defpath_keys = set(self._path_key(x) for x in self._defpath)
 		satisfied_consumer_keys = set()
 		if soname_node is not None:
-			if exclude_providers is not None:
+			if exclude_providers is not None or not greedy:
 				relevant_dir_keys = set()
 				for provider_key in soname_node.providers:
+					if not greedy and provider_key == obj_key:
+						continue
 					provider_objs = self._obj_properties[provider_key].alt_paths
 					for p in provider_objs:
 						provider_excluded = False
-						for excluded_provider_isowner in exclude_providers:
-							if excluded_provider_isowner(p):
-								provider_excluded = True
-								break
+						if exclude_providers is not None:
+							for excluded_provider_isowner in exclude_providers:
+								if excluded_provider_isowner(p):
+									provider_excluded = True
+									break
 						if not provider_excluded:
 							# This provider is not excluded. It will
 							# satisfy a consumer of this soname if it
